function EtManager(spaceOrder) {
	this.spaceOrder = spaceOrder;
	this.channel	= {};
	this.spaces		= {};
	this.ads		= [];
}

EtManager.prototype.hasSpaces = function() {
	for(var i in this.spaces) {
		return true;
	}
	return false;
};

EtManager.prototype.hasAds = function() {
	return this.ads.length > 0;
};

EtManager.prototype.registerAdSpace = function(name, width, height, padding, sizes, dom, fallback) {
	this.spaces[name] = {
		'ads'		: 0,
		'max'		: 0,
		'min'		: 0,
		'name'		: name,
		'width'		: width,
		'height'	: height,
		'padding'	: padding,
		'sizes'		: sizes,
		'fallback'	: fallback,
		'objects'	: {
			'frame'		: dom,
			'tile'		: dom.find('.nexus-shop-ad-tile').remove()
		}
	};
	for(var i in sizes) {
		this.spaces[name].max = Math.max(i, this.spaces[name].max);
		this.spaces[name].min = Math.min(i, this.spaces[name].min);
	}
};
EtManager.prototype.add = function(ad) {
	this.ads.push(ad);
};

EtManager.prototype.layout = function() {
	
	// filter available spaces
	
	var spaceOrder = [];
	for(var i=0;i<this.spaceOrder.length;i++) {
		if(this.spaces[this.spaceOrder[i]]) {
			spaceOrder.push(this.spaceOrder[i]);
		}
	}
	this.spaceOrder = spaceOrder;
	
	if(this.ads.length == 0) {
		for(var i in this.spaces) {
			$(this.spaces[i].objects.frame[0].parentNode).hide();
		}
	}
	
	// calc optimal distribution
	
	var dist = [];
	for(var i=0;i<this.ads.length;i++) {
		var s = i%this.spaceOrder.length;
		if(!dist[s])
			dist.push(0);
		dist[s]++;
	}
	
	for(var i=0;i<dist.length;i++) {
		var spaceName = this.spaceOrder[i];
		var space = this.spaces[spaceName];
		var d = dist[i];
		while(!space.sizes[d] && d < space.max) {
			d++;
		}
		while(!space.sizes[d] && d > space.min) {
			d--;
		}
		if(i+1 < dist.length) {
			dist[i+1] += (dist[i]-d);
		}
		dist[i] = d;
	}
	
	// deliver
	
	var a = 0;
	for(var s=0;s<this.spaceOrder.length;s++) {
		
		var spaceName = this.spaceOrder[s];
		var space = this.spaces[spaceName];
		var size = space.sizes[dist[s]];
		
		// all ads delivered, stop
		if(a == this.ads.length) {
			$(space.objects.frame[0].parentNode).hide();
			continue;
		}
		
		var cols = Math.round(space.width/size.tileWidth);
		for(var ads=0;ads<dist[s];ads++) {
			var ad = this.ads[a++];

			space.objects.frame.css('width', space.width+'px');
			space.objects.frame.css('height', space.height+'px');

			var entry = space.objects.tile.clone();

			entry.css('font-size', size.font);
			entry.find('.nexus-shop-ad-title').text(ad.title);
			entry.attr('title', ad.title);
			if(ad.sub)
				entry.find('.nexus-shop-ad-sub').text(ad.sub);
			else
				entry.find('.nexus-shop-ad-sub').hide();
			if(ad.price)
				entry.find('.nexus-shop-ad-price').text(ad.price);
			else
				entry.find('.nexus-shop-ad-price').hide();
			entry.attr('href', ad.link);
			
			(function(entry,spaceName,id) {				
				entry.click(function() { 
					_gaq.push(['_trackEvent', 'Shop Ad '+spaceName, 'Click', 'Shop Ad '+spaceName+': '+id]);
					return true; 
				});
			})(entry,spaceName,ad.id);

			// img
			var img = entry.find('.nexus-shop-ad-image img');
			var ic = entry.find('.nexus-shop-ad-image');
			img.attr('src', ad.image);

			var ih = size.imageHeight;
			var iw = ih/ad.aspect;
			if(iw > size.imageWidth) {
				iw = size.imageWidth;
				ih = iw*ad.aspect;
			}

			ic.css('width', size.imageWidth+'px');
			ic.css('height', size.imageHeight+'px');
			img.css('width', Math.round(iw)+'px');
			img.css('height',Math.round(ih)+'px');

			// pos
			var x = ads%cols;
			var y = Math.floor(ads/cols);

			entry.css('width', (size.tileWidth-2*space.padding)+'px');
			entry.css('height', (size.tileHeight-2*space.padding)+'px');
			entry.css('left', (Math.round(size.tileWidth*x)+space.padding)+'px');
			entry.css('top', (Math.round(size.tileHeight*y)+space.padding)+'px');

			space.objects.frame.append(entry);
		}
	}
};

EtManager.prototype.addChannel = function(name, url) {
	if(url) {
		this.channel[name] = {
			'url'	: url,
			'state' : false
		};
	}
};

EtManager.prototype.init = function() {
	for(var c in this.channel) {
		$('body').append($('<script type="text/javascript" src="'+this.channel[c].url+'"></script>'));
	}
};

EtManager.prototype.registerChannel = function(name) {
	this.channel[name].state = true;
	
	// start layout when last channel arrived
	var layout = true;
	for(var c in this.channel) {
		if(!this.channel[c].state) {
			layout = false;
		}
	}
	
	if(layout) {
		// randomize array
		var ads = this.ads;
		this.ads = [];
		while(ads.length > 0) {
			var i = Math.round(Math.random()*(ads.length-1));
			this.ads.push(ads[i]);			
			ads = ads.slice(0,i).concat(ads.slice(i+1,ads.length));
		}
		this.layout();
	}
};